@@ -2,8 +2,9 @@
 Name               : check_syntax.c
 Author             : Marvin Raaijmakers
 Description        : Checks if the syntax of a keyboard file is valid
-Date of last change: 04-Jul-2006
-History            : 24-Jun-2006 Added checks for keycodes and scancodes
+Date of last change: 17-Sep-2006
+History            : 17-Sep-2006 Added support for ACPI hotkeys
+                     24-Jun-2006 Added checks for keycodes and scancodes
 
     Copyright (C) 2006 Marvin Raaijmakers
 
@@ -45,10 +46,12 @@ static Boolean check_key_settings (	XmlContent		*keylist_element,
 					XmlElementType		*keyname_type,
 					XmlElementType		*keycode_type,
 					XmlElementType		*scancode_type,
+					XmlElementType		*acpi_event_descr_type,
 					XmlElementType		*default_action_type,
 					XmlElementType		*pluginname_type,
 					XmlElementType		*pluginfunction_type,
 					XmlAttributeName	*action_type_attrname,
+					XmlAttributeName	*key_type_attrname,
 					unsigned int		*num_keys,
 					char			*key_names[MAX_NUM_KEYS],
 					Boolean			keycode_used[NUM_KEYCODES],
@@ -115,10 +118,12 @@ check_key_settings (	XmlContent		*keylist_element,
 			XmlElementType		*keyname_type,
 			XmlElementType		*keycode_type,
 			XmlElementType		*scancode_type,
+			XmlElementType		*acpi_event_descr_type,
 			XmlElementType		*default_action_type,
 			XmlElementType		*pluginname_type,
 			XmlElementType		*pluginfunction_type,
 			XmlAttributeName	*action_type_attrname,
+			XmlAttributeName	*key_type_attrname,
 			unsigned int		*num_keys,
 			char			*key_names[MAX_NUM_KEYS],
 			Boolean			keycode_used[NUM_KEYCODES],
@@ -131,10 +136,12 @@ Input:
 	keyname_type		- The XmlElementType for the key-name element
 	keycode_type		- The XmlElementType for the keycode element
 	scancode_type		- The XmlElementType for the scancode element
+	acpi_event_descr_type	- The XmlElementType for the event-descr element
 	default_action_type	- The XmlElementType for the default-action element
 	pluginname_type		- The XmlElementType for the plugin-name element
 	pluginfunction_type	- The XmlElementType for the plugin-function element
 	action_type_attrname	- The XmlElementName for the action-type attribute
+	key_type_attrname	- The XmlElementName for the key-type attribute
 	num_keys		- The number of elements used in 'key_names'
 	key_names		- Array of pointers to key names that are in the
 				  keyboard file.
@@ -161,24 +168,31 @@ Desciption:
 */
 {
 	XmlContent	*key_element,
+			*event_descr_element,
 			*keyname_element,
 			*keycode_element,
 			*scancode_element,
 			*default_action_element;
 	unsigned int	i, keycode;
 	char		*key_name,
+			*key_type_str,
 			*end_pointer,
 			*keycode_string,
 			*scancode_string;
+	Boolean		is_acpi_hotkey;
 	
 	*valid = TRUE;
 	key_element = XmlGetElementOfType(XmlElementContentList(keylist_element), key_type, index);
 	if (key_element)
 	{
+		key_type_str = XmlGetAttributeValue (key_type_attrname, key_element);
+		is_acpi_hotkey = (key_type_str && !strcmp(key_type_str, ATTR_VAL_ACPIHOTKEY));
 		if ((keyname_element = XmlGetElementOfType(XmlElementContentList(key_element), keyname_type, 0)) == NULL ||
 		    (default_action_element = XmlGetElementOfType(XmlElementContentList(key_element), default_action_type, 0)) == NULL ||
 		    (keycode_element = XmlGetElementOfType(XmlElementContentList(key_element), keycode_type, 0)) == NULL ||
-		    (scancode_element = XmlGetElementOfType(XmlElementContentList(key_element), scancode_type, 0)) == NULL )
+		    (is_acpi_hotkey ?
+		       (event_descr_element = XmlGetElementOfType(XmlElementContentList(key_element), acpi_event_descr_type, 0)) == NULL :
+		       (scancode_element = XmlGetElementOfType(XmlElementContentList(key_element), scancode_type, 0)) == NULL) )
 		{
 			KTSetErrorMsg (_("The keyboard file contains an incomplete key element."), "");
 			*valid = FALSE;
@@ -209,17 +223,22 @@ Desciption:
 			}
 			if (*valid)
 			{
-				/** Check the scancode **/
-				scancode_string = XmlGetElementString(scancode_element, "");
-				/* Convert the string of the scancode to a integer */
-				strtol (scancode_string, &end_pointer, 0);
-				/* If the scancode string is not a vallid scancode */
-				if (*end_pointer != '\0')
+				if (!is_acpi_hotkey)
 				{
-					KTSetErrorMsg (_("'%s' is an invalid scancode."), scancode_string);
-					*valid = FALSE;
+					/** Check the scancode **/
+					scancode_string = XmlGetElementString(scancode_element, "");
+					/* Convert the string of the scancode to a integer */
+					strtol (scancode_string, &end_pointer, 0);
+					/* If the scancode string is not a vallid scancode */
+					if (*end_pointer != '\0')
+					{
+						KTSetErrorMsg (_("'%s' is an invalid scancode."), scancode_string);
+						*valid = FALSE;
+					}
+					XmlFree (scancode_string);
 				}
-				else
+				
+				if (*valid)
 				{
 					/** Check the keycode **/
 					keycode_string = XmlGetElementString (keycode_element, "");
@@ -249,17 +268,17 @@ Desciption:
 						*valid = FALSE;
 					}
 					XmlFree (keycode_string);
+					
+					if (*valid && *num_keys < MAX_NUM_KEYS-1)
+					{
+						key_names[*num_keys] = key_name;
+						(*num_keys)++;
+					}
+					else
+					{
+						XmlFree (key_name);
+					}
 				}
-				XmlFree (scancode_string);
-			}
-			if (*valid && *num_keys < MAX_NUM_KEYS-1)
-			{
-				key_names[*num_keys] = key_name;
-				(*num_keys)++;
-			}
-			else
-			{
-				XmlFree (key_name);
 			}
 		}
 	}
@@ -392,6 +411,7 @@ Desciption:
 				*key_type,
 				*keyname_type,
 				*scancode_type,
+				*event_descr_type,
 				*keycode_type,
 				*default_action_type,
 				*pluginname_type,
@@ -400,13 +420,18 @@ Desciption:
 				*keyboard_info_type,
 				*model_type,
 				*manufacturer_type;
-	XmlAttributeName	*action_type_attrname;
+	XmlAttributeName	*action_type_attrname,
+				*key_type_attrname;
 	Boolean			error, valid;
 	unsigned int		index, num_keys;
 	char			*key_names[MAX_NUM_KEYS];
 	Boolean			keycode_used[NUM_KEYCODES] = {[0 ... NUM_KEYCODES-1] = 0};
 	
 	error = FALSE;
+	
+	event_descr_type = XmlGetElementType (ELEM_TYPE_ACPI_EVENT_DESCR, keyboard_file, TRUE);
+	scancode_type = XmlGetElementType (ELEM_TYPE_SCANCODE, keyboard_file, TRUE);
+	
 	if ((keyboard_type = XmlGetElementType (ELEM_TYPE_KEYBOARD, keyboard_file, FALSE)) == NULL ||
 	    (keyboard_element = XmlGetElementOfType (XmlDocumentContentList(keyboard_file), keyboard_type, 0)) == NULL)
 	{
@@ -466,11 +491,6 @@ Desciption:
 		KTSetErrorMsg (_("The keyboard file does not contain any %s element."), ELEM_TYPE_KEYNAME);
 		error = TRUE;
 	}
-	else if ((scancode_type = XmlGetElementType(ELEM_TYPE_SCANCODE, keyboard_file, FALSE)) == NULL)
-	{
-		KTSetErrorMsg (_("The keyboard file does not contain any %s element."), ELEM_TYPE_SCANCODE);
-		error = TRUE;
-	}
 	else if ((keycode_type = XmlGetElementType(ELEM_TYPE_KEYCODE, keyboard_file, FALSE)) == NULL)
 	{
 		KTSetErrorMsg (_("The keyboard file does not contain any %s element."), ELEM_TYPE_KEYCODE);
@@ -479,6 +499,7 @@ Desciption:
 	else
 	{
 		action_type_attrname = XmlGetAttributeName (ATTR_NAME_ACTIONTYPE, keyboard_file, TRUE);
+		key_type_attrname = XmlGetAttributeName (ATTR_NAME_KEYTYPE, keyboard_file, TRUE);
 		/* Create the element types "plugin-name" and "plugin-function" if they
 		 * do not appear in the document, because these element do not necessary
 		 * appear in the document */
@@ -488,9 +509,9 @@ Desciption:
 		num_keys = 0;
 		for (index = 0;
 		     check_key_settings (keylist_element, index, key_type, keyname_type, keycode_type,
-		                         scancode_type, default_action_type, pluginname_type,
-		                         pluginfunction_type, action_type_attrname, &num_keys,
-		                         key_names, keycode_used, &valid);
+		                         scancode_type, event_descr_type, default_action_type,
+		                         pluginname_type, pluginfunction_type, action_type_attrname,
+		                         key_type_attrname, &num_keys, key_names, keycode_used, &valid);
 		     index++)
 		{
 			if (!valid)
