@@ -2,8 +2,11 @@
 Name               : config.c
 Author             : Marvin Raaijmakers
 Description        : Reads and writes the configuration files
-Date of last change: 19-Mar-2006
-History            : 19-Mar-2006 get_current_keyboard() exists quietly if
+Date of last change: 30-Aug-2006
+History            : 30-Aug-2006 In read_key_settings strcmp, for comparing the
+                                 keycode with "VOLUMEUP" or "VOLUMEDOWN", by
+                                 strcasecmp
+                     19-Mar-2006 get_current_keyboard() exists quietly if
                                  CURRENT_KEYBOARD_FILE does not exist
                      26-Feb-2006 auto_repeat_on will be set by read_key_settings()
                      12-Jan-2006 read_key_settings_action() now continues after
@@ -38,7 +41,7 @@ History            : 19-Mar-2006 get_current_keyboard() exists quietly if
 #include <mxml.h>
 
 #include <string_to_keycode.h>
-#include <keytouchd.h> 
+#include <keytouchd.h>
 
 static KTKeySettings *read_key_settings (	XmlContent		*keylist_element,
 						int			index,
@@ -120,8 +123,8 @@ Desciption:
 		KTError ("The keyboard file contains an invalid keycode: %s.", keycode_string);
 		exit (EXIT_FAILURE);
 	}
-	key_settings->auto_repeat_on = !strcmp("VOLUMEUP", keycode_string) ||
-	                               !strcmp("VOLUMEDOWN", keycode_string);
+	key_settings->auto_repeat_on = !strcasecmp("VOLUMEUP", keycode_string) ||
+	                               !strcasecmp("VOLUMEDOWN", keycode_string);
 	XmlFree (keycode_string);
 	/* Initialize the remaining elements: */
 	key_settings->next = NULL;
